/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { enterWith } from '../../support/cypress.util';
import { convertToDataTestId } from '../../support/tech.util';

export class VorgangSearchE2EComponent {
  private readonly locatorButton: string = 'search-button';
  private readonly locatorInput: string = 'search-input';
  private readonly locatorForm: string = 'search-form';
  private readonly locatorSearchPreviewList: string = 'search-preview-list';
  private readonly locatorClearButton: string = 'clear-button';

  private readonly locatorRoot: string = 'vorgang-search';

  public getRoot() {
    return cy.getTestElement(this.locatorRoot);
  }

  public getInput(): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.locatorInput);
  }

  public getSearchButton() {
    return cy.getTestElement(this.locatorButton);
  }

  public getForm() {
    return cy.getTestElement(this.locatorForm);
  }

  public getVorgangSearchPreviewList() {
    return cy.getTestElement(this.locatorSearchPreviewList);
  }

  public getVorgangSearchPreviewListItem(
    vorgangName: string,
  ): VorgangSearchPreviewListItemE2EComponent {
    return new VorgangSearchPreviewListItemE2EComponent(vorgangName);
  }

  public getClearButton() {
    return cy.getTestElement(this.locatorClearButton);
  }

  public doSearch(searchString: string): void {
    enterWith(this.getInput(), searchString, 300);
  }
}

export class VorgangSearchPreviewListItemE2EComponent {
  private readonly locatorCreatedAt: string = 'preview-list-item-created-at';
  private readonly locatorVorgangName: string = 'preview-list-item-vorgang-name';
  private readonly locatorVorgangAktenzeichen: string = 'preview-list-item-vorgang-aktenzeichen';
  private readonly locatorVorgangNummer: string = 'vorgang-nummer';

  private locatorRoot: string;

  constructor(vorgangName: string) {
    this.locatorRoot = 'search-preview-list-item-' + convertToDataTestId(vorgangName);
  }

  public getRoot() {
    return cy.getTestElement(this.locatorRoot);
  }

  public getCreatedAt() {
    return this.getRoot().getTestElement(this.locatorCreatedAt);
  }

  public getVorgangName() {
    return this.getRoot().getTestElement(this.locatorVorgangName);
  }

  public getVorgangAktenzeichen() {
    return this.getRoot().getTestElement(this.locatorVorgangAktenzeichen);
  }

  public getVorgangNummer() {
    return this.getRoot().getTestElement(this.locatorVorgangNummer);
  }
}
