/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { VorgangFormularButtonsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import {
  VorgangE2E,
  VorgangMessagesE2E,
  VorgangStatusE2E,
  vorgangStatusLabelE2E,
} from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections, wait } from '../../../support/cypress-helper';
import { contains, exist, haveText, notExist } from '../../../support/cypress.util';
import { loginAsEmil } from '../../../support/user-util';
import { buildVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

describe('Vorgang löschen anfordern', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
  const snackBar: SnackBarE2EComponent = mainPage.getSnackBar();

  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangFormularButtons: VorgangFormularButtonsE2EComponent =
    vorgangPage.getFormularButtons();

  const vorgangAbgeschlossenLoeschenAnfordern: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'DoAbgeschlossenLoeschenAnfordern'),
    status: VorgangStatusE2E.ABGESCHLOSSEN,
  };
  const vorgangAbgeschlossenLoeschenAnfordernRevoke: VorgangE2E = {
    ...buildVorgang(objectIds[1], 'DoAbgeschlossenRevokeLoeschenAnfordern'),
    status: VorgangStatusE2E.ABGESCHLOSSEN,
  };
  const vorgangVerworfenLoeschenAnfordern: VorgangE2E = {
    ...buildVorgang(objectIds[2], 'DoVerworfenLoeschenAnfordern'),
    status: VorgangStatusE2E.VERWORFEN,
  };
  const vorgangVerworfenLoeschenAnfordernRevoke: VorgangE2E = {
    ...buildVorgang(objectIds[3], 'DoVerworfenRevokeLoeschenAnfordern'),
    status: VorgangStatusE2E.VERWORFEN,
  };

  before(() => {
    initVorgaenge([
      vorgangAbgeschlossenLoeschenAnfordern,
      vorgangAbgeschlossenLoeschenAnfordernRevoke,
      vorgangVerworfenLoeschenAnfordern,
      vorgangVerworfenLoeschenAnfordernRevoke,
    ]);

    loginAsEmil();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('Abgeschlossen: löschen anfordern', () => {
    it('Open Vorgang-Detail-Page', () => {
      vorgangList.getListItem(vorgangAbgeschlossenLoeschenAnfordern.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    describe('by button', () => {
      it('should have status Abgeschlossen', () => {
        haveText(
          vorgangPage.getVorgangDetailHeader().getStatus(),
          vorgangStatusLabelE2E[vorgangAbgeschlossenLoeschenAnfordern.status],
        );
      });

      it('should show snackBar message', () => {
        vorgangFormularButtons.getLoeschenAnfordernButton().click();
        waitForSpinnerToDisappear();

        exist(snackBar.getCloseButton());
        contains(snackBar.getMessage(), VorgangMessagesE2E.LOESCHEN_ANFORDERN);
      });

      it('should close snackBar', () => {
        snackBar.getCloseButton().click();

        notExist(snackBar.getMessage());
      });

      it('should have status zu Löschen', () => {
        haveText(
          vorgangPage.getVorgangDetailHeader().getStatus(),
          vorgangStatusLabelE2E[VorgangStatusE2E.ZU_LOESCHEN],
        );
      });

      it('back to vorgang list', () => {
        vorgangPage.getSubnavigation().getBackButton().click();
        waitForSpinnerToDisappear();

        exist(vorgangList.getRoot());
      });

      it('should have status zu Löschen', () => {
        haveText(
          vorgangList.getListItem(vorgangAbgeschlossenLoeschenAnfordern.name).getStatus(),
          vorgangStatusLabelE2E[VorgangStatusE2E.ZU_LOESCHEN],
        );
      });
    });
  });

  describe('Abgeschlossen: löschen anfordern und rückgängig machen', () => {
    it('Open Vorgang-Detail-Page', () => {
      vorgangList.getListItem(vorgangAbgeschlossenLoeschenAnfordernRevoke.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    describe('by icon-button', () => {
      it('should have status Abgeschlossen', () => {
        haveText(
          vorgangPage.getVorgangDetailHeader().getStatus(),
          vorgangStatusLabelE2E[vorgangAbgeschlossenLoeschenAnfordernRevoke.status],
        );
      });

      it('should show snackBar message', () => {
        vorgangPage.getSubnavigation().getLoeschenAnfordernByIconButton().click();
        waitForSpinnerToDisappear();

        exist(snackBar.getCloseButton());
        contains(snackBar.getMessage(), VorgangMessagesE2E.LOESCHEN_ANFORDERN);
      });

      it('should close snackbar on revoke', () => {
        snackBar.getRevokeButton().click();
        waitForSpinnerToDisappear();

        notExist(snackBar.getMessage());
      });

      it('should show status Abgeschlossen', () => {
        notExist(vorgangPage.getProgressBar());
        wait(2000, 'Statusänderung flaky');
        cy.scrollTo(0, -500);

        haveText(
          vorgangPage.getVorgangDetailHeader().getStatus(),
          vorgangStatusLabelE2E[VorgangStatusE2E.ABGESCHLOSSEN],
        );
      });

      it('back to vorgang list', () => {
        vorgangPage.getSubnavigation().getBackButton().click();
        waitForSpinnerToDisappear();

        exist(vorgangList.getRoot());
      });

      it('should have status Abgeschlossen', () => {
        haveText(
          vorgangList.getListItem(vorgangAbgeschlossenLoeschenAnfordernRevoke.name).getStatus(),
          vorgangStatusLabelE2E[VorgangStatusE2E.ABGESCHLOSSEN],
        );
      });
    });
  });

  describe('Verworfen: löschen anfordern', () => {
    it('Open Vorgang-Detail-Page', () => {
      vorgangList.getListItem(vorgangVerworfenLoeschenAnfordern.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    describe('by button', () => {
      it('should have status Verworfen', () => {
        haveText(
          vorgangPage.getVorgangDetailHeader().getStatus(),
          vorgangStatusLabelE2E[vorgangVerworfenLoeschenAnfordern.status],
        );
      });

      it('should show snackBar message', () => {
        vorgangFormularButtons.getLoeschenAnfordernButton().click();
        waitForSpinnerToDisappear();

        exist(snackBar.getCloseButton());
        contains(snackBar.getMessage(), VorgangMessagesE2E.LOESCHEN_ANFORDERN);
      });

      it('should close snackBar', () => {
        snackBar.getCloseButton().click();

        notExist(snackBar.getMessage());
      });

      it('should have status zu Löschen', () => {
        haveText(
          vorgangPage.getVorgangDetailHeader().getStatus(),
          vorgangStatusLabelE2E[VorgangStatusE2E.ZU_LOESCHEN],
        );
      });

      it('back to vorgang list', () => {
        vorgangPage.getSubnavigation().getBackButton().click();
        waitForSpinnerToDisappear();

        exist(vorgangList.getRoot());
      });

      it('should have status zu Löschen', () => {
        haveText(
          vorgangList.getListItem(vorgangVerworfenLoeschenAnfordern.name).getStatus(),
          vorgangStatusLabelE2E[VorgangStatusE2E.ZU_LOESCHEN],
        );
      });
    });
  });

  describe('Verworfen: löschen anfordern und rückgängig machen', () => {
    it('Open Vorgang-Detail-Page', () => {
      vorgangList.getListItem(vorgangVerworfenLoeschenAnfordernRevoke.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    describe('by icon-button', () => {
      it('should have status Verworfen', () => {
        haveText(
          vorgangPage.getVorgangDetailHeader().getStatus(),
          vorgangStatusLabelE2E[vorgangVerworfenLoeschenAnfordernRevoke.status],
        );
      });

      it('should show snackBar message', () => {
        vorgangPage.getSubnavigation().getLoeschenAnfordernByIconButton().click();
        waitForSpinnerToDisappear();

        exist(snackBar.getCloseButton());
        contains(snackBar.getMessage(), VorgangMessagesE2E.LOESCHEN_ANFORDERN);
      });

      it('should close snackbar on revoke', () => {
        snackBar.getRevokeButton().click();
        waitForSpinnerToDisappear();

        notExist(snackBar.getMessage());
      });

      it('should show status Verworfen', () => {
        notExist(vorgangPage.getProgressBar());

        wait(2000, 'Statusänderung im UI nicht schnell genug');
        cy.scrollTo(0, -500);

        vorgangPage.getVorgangDetailHeader().getStatus();
        haveText(
          vorgangPage.getVorgangDetailHeader().getStatus(),
          vorgangStatusLabelE2E[VorgangStatusE2E.VERWORFEN],
        );
      });

      it('back to vorgang list', () => {
        vorgangPage.getSubnavigation().getBackButton().click();
        waitForSpinnerToDisappear();

        exist(vorgangList.getRoot());
      });

      it('should have status Verworfen', () => {
        haveText(
          vorgangList.getListItem(vorgangVerworfenLoeschenAnfordernRevoke.name).getStatus(),
          vorgangStatusLabelE2E[VorgangStatusE2E.VERWORFEN],
        );
      });
    });
  });
});
