import {
  StateResource,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { SnackBarService } from '@alfa-client/ui';
import { Type } from '@angular/core';
import { fakeAsync, tick } from '@angular/core/testing';
import { cold } from 'jest-marbles';
import { singleCold, singleHot } from 'libs/tech-shared/test/marbles';
import { Observable, of } from 'rxjs';
import { createPostfachResource, createPostfachSettingItem } from '../../../test/postfach/postfach';
import { PostfachResourceService } from './postfach-resource.service';
import { PostfachResource, PostfachSettingsItem } from './postfach.model';
import { PostfachService } from './postfach.service';

describe('PostfachService', () => {
  let service: PostfachService;
  let resourceService: Mock<PostfachResourceService>;
  const snackbarService: Mock<SnackBarService> = mock(SnackBarService);

  beforeEach(() => {
    resourceService = mockResourceService(PostfachResourceService);

    service = new PostfachService(useFromMock(resourceService), useFromMock(snackbarService));
  });

  it('should create', () => {
    expect(service).toBeTruthy();
  });

  describe('get', () => {
    const postfachResource: PostfachResource = createPostfachResource();
    const postfachStateResource: StateResource<PostfachResource> =
      createStateResource(postfachResource);

    it('should call resourceservice get', () => {
      service.get();

      expect(resourceService.get).toHaveBeenCalled();
    });

    it('should reurn value', () => {
      resourceService.get.mockReturnValue(singleCold(postfachStateResource));

      const returnedPostfachResource: Observable<StateResource<PostfachResource>> = service.get();

      expect(returnedPostfachResource).toBeObservable(singleHot(postfachStateResource));
    });
  });

  describe('save', () => {
    const postfachSettingsItem: PostfachSettingsItem = createPostfachSettingItem();
    const postfachResource: PostfachResource = createPostfachResource();
    const postfachStateResource: StateResource<PostfachResource> =
      createStateResource(postfachResource);

    beforeEach(() => {});

    it('should call resourceService', () => {
      resourceService.save.mockReturnValue(of(postfachStateResource));

      service.save(postfachSettingsItem.settingBody);

      expect(resourceService.save).toHaveBeenCalledWith(postfachSettingsItem);
    });

    it('should return saved value', () => {
      resourceService.save.mockReturnValue(singleCold(postfachStateResource, '-a'));

      const savedPostfach: Observable<StateResource<PostfachResource>> = service.save(
        postfachResource.settingBody,
      );

      expect(savedPostfach).toBeObservable(
        cold('ab', {
          a: createEmptyStateResource<PostfachResource>(true),
          b: postfachStateResource,
        }),
      );
    });

    it('should show message in snackbar', fakeAsync(() => {
      resourceService.save.mockReturnValue(of(postfachStateResource));
      const savedPostfach: Observable<StateResource<PostfachResource>> = service.save(
        postfachResource.settingBody,
      );

      savedPostfach.subscribe();
      tick();

      expect(snackbarService.showInfo).toHaveBeenCalledWith(
        'Die Signatur wurde erfolgreich gespeichert.',
      );
    }));
  });
});

export function mockResourceService<T>(service: Type<T>): Mock<T> {
  return <Mock<T>>{ ...mock(service), get: jest.fn(), save: jest.fn() };
}
