import { ConvertForDataTestPipe } from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule } from '@angular/forms';
import { RouterTestingModule } from '@angular/router/testing';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { NavigationItemComponent } from './navigation-item.component';

describe('NavigationItemComponent', () => {
  let component: NavigationItemComponent;
  let fixture: ComponentFixture<NavigationItemComponent>;

  const anchorLink = getDataTestIdOf('anchor');
  const navigationLabelName = 'test';
  const image = getDataTestIdOf('image-' + navigationLabelName);
  const label = getDataTestIdOf('navigation-label-' + navigationLabelName);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [NavigationItemComponent, ConvertForDataTestPipe],
      imports: [ReactiveFormsModule, RouterTestingModule],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(NavigationItemComponent);
    component = fixture.componentInstance;
    component.name = 'test';
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should use imageSrc input', () => {
    const imageSrc = 'imagesrc';
    component.imageSrc = imageSrc;

    fixture.detectChanges();

    const imageElement: HTMLImageElement = getElementFromFixture(fixture, image);
    expect(imageElement.getAttribute('src')).toBe(imageSrc);
  });

  it('should use router link', () => {
    const link = '/testroute';
    component.link = link;

    fixture.detectChanges();

    const anchorElement: HTMLAnchorElement = getElementFromFixture(fixture, anchorLink);
    expect(anchorElement.getAttribute('ng-reflect-router-link')).toBe(link);
  });

  describe('image name', () => {
    it('should be used for alt', () => {
      const imageElement = getElementFromFixture(fixture, image);
      expect(imageElement.alt).toBe(navigationLabelName);
    });

    it('should be used for navigation-label', () => {
      const labelElement = getElementFromFixture(fixture, label);
      expect(labelElement.textContent).toBe(navigationLabelName);
    });
  });
});
