import { EMPTY_STRING } from '@alfa-client/tech-shared';
import { createOrganisationseinheitError, createUser } from '../../../test/user/user';
import { OrganisationseinheitError, OrganisationseinheitErrorType, User } from './user.model';
import { KEYCLOAK_ERROR_MESSAGES, getOrganisationseinheitErrorMessage, sortUsersByLastName } from './user.util';

describe('user util', () => {
  describe('get organisationseinheit error message', () => {
    it('should map known error message', () => {
      const nameConflictError: OrganisationseinheitError = createOrganisationseinheitError(
        OrganisationseinheitErrorType.NAME_CONFLICT,
      );
      const expectedMessage: string = KEYCLOAK_ERROR_MESSAGES[nameConflictError.errorType];

      const message: string = getOrganisationseinheitErrorMessage(nameConflictError);

      expect(message).toEqual(expectedMessage);
    });

    it('should map unknown error message to empty string', () => {
      const nameConflictError: OrganisationseinheitError = createOrganisationseinheitError(null);

      const message: string = getOrganisationseinheitErrorMessage(nameConflictError);

      expect(message).toEqual(EMPTY_STRING);
    });
  });

  describe('sort users by last name', () => {
    const users: User[] = [
      { ...createUser(), lastName: 'Müller' },
      { ...createUser(), lastName: 'Schmidt' },
      { ...createUser(), lastName: 'Anders' },
    ];

    it('shoud sort users by last name alphabetically ', () => {
      const sortedUsers: User[] = sortUsersByLastName(users);

      expect(sortedUsers[0].lastName).toBe('Anders');
      expect(sortedUsers[1].lastName).toBe('Müller');
      expect(sortedUsers[2].lastName).toBe('Schmidt');
    });
  });
});
