/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { faker } from '@faker-js/faker';
import { ApiRootLinkRel, ApiRootResource } from '@alfa-client/api-root-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { HttpService, ResourceFactory, ResourceUri } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { Environment } from 'libs/environment-shared/src/lib/environment.model';
import { createEnvironment } from 'libs/environment-shared/test/environment';
import { ApiRootRepository } from './api-root.repository';

describe('ApiRoot Repository', () => {
  let repository: ApiRootRepository;
  let httpService: Mock<HttpService>;
  let resourceFactory = mock(ResourceFactory);
  let resourceWrapper = { post: jest.fn() };

  const environment: Environment = createEnvironment();

  beforeEach(() => {
    httpService = mock(HttpService);

    repository = new ApiRootRepository(
      useFromMock(httpService),
      useFromMock(resourceFactory),
      environment,
    );

    resourceFactory.from.mockReturnValue(resourceWrapper);
  });

  describe('loadApiRoot', () => {
    it('should call httpService', () => {
      repository.loadApiRoot();

      expect(httpService.get).toHaveBeenCalledWith(environment.remoteHost);
    });
  });

  describe('downloadToken', () => {
    const resource: ApiRootResource = createApiRootResource();
    const uri: ResourceUri = faker.random.word();

    beforeEach(() => {
      resourceWrapper.post.mockReturnValue(hot('a', { a: resource }));
    });

    it('should call resourceFactory', () => {
      repository.downloadToken(resource, uri);

      expect(resourceFactory.from).toHaveBeenCalledWith(resource);
    });

    it('should call resourceWrapper with link and body', () => {
      repository.downloadToken(resource, uri);

      expect(resourceWrapper.post).toHaveBeenCalledWith(ApiRootLinkRel.DOWNLOAD_TOKEN, {
        fileId: uri,
      });
    });

    it('should return result', () => {
      let result = repository.downloadToken(resource, uri);

      expect(result).not.toBeNull();
      expect(result).toBeObservable(cold('a', { a: resource }));
    });
  });
});
