import { BescheidLinkRel, BescheidResource, BescheidService } from '@alfa-client/bescheid-shared';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { SimpleChanges } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createBescheidResource } from 'libs/bescheid-shared/src/test/bescheid';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { BescheidInVorgangContainerComponent } from './bescheid-in-vorgang-container.component';
import { BescheidInVorgangComponent } from './bescheid-in-vorgang/bescheid-in-vorgang.component';

describe('BescheidInVorgangContainerComponent', () => {
  let component: BescheidInVorgangContainerComponent;
  let fixture: ComponentFixture<BescheidInVorgangContainerComponent>;

  let bescheidService: Mock<BescheidService>;
  const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();

  const noChanges: SimpleChanges = {};
  const changes: SimpleChanges = <any>{ vorgang: vorgang };

  const bescheidResource: BescheidResource = createBescheidResource();
  const bescheidStateResource: StateResource<BescheidResource> =
    createStateResource(bescheidResource);

  beforeEach(async () => {
    bescheidService = mock(BescheidService);
    bescheidService.getBescheidDocumentFile = jest.fn();
    bescheidService.loadBescheidDocument = jest.fn();
    bescheidService.getBescheidDraft.mockReturnValue(of(bescheidStateResource));

    await TestBed.configureTestingModule({
      declarations: [
        BescheidInVorgangContainerComponent,
        MockComponent(BescheidInVorgangComponent),
      ],
      providers: [{ provide: BescheidService, useValue: bescheidService }],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidInVorgangContainerComponent);
    component = fixture.componentInstance;
    component.vorgang = vorgang;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('hasBescheidDraft', () => {
    it('should return false', () => {
      component.vorgang = vorgang;

      expect(component.hasBescheidDraft).toBeFalsy();
    });

    it('should return true', () => {
      component.vorgang = createVorgangWithEingangResource([
        VorgangWithEingangLinkRel.BESCHEID_DRAFT,
      ]);

      expect(component.hasBescheidDraft).toBeTruthy();
    });
  });

  describe('onChanges', () => {
    it('should not call getBescheid', () => {
      component.getBescheid = jest.fn();

      component.ngOnChanges(noChanges);

      expect(component.getBescheid).not.toHaveBeenCalled();
    });

    it('should call getBescheid', () => {
      component.getBescheid = jest.fn();

      component.ngOnChanges(changes);

      expect(component.getBescheid).toHaveBeenCalled();
    });

    it('should not call bescheidService.getBescheidDocumentFile', () => {
      component.ngOnChanges(noChanges);

      expect(bescheidService.getBescheidDocumentFile).not.toHaveBeenCalled();
    });

    it('should call bescheidService.getBescheidDocumentFile', () => {
      component.ngOnChanges(changes);

      expect(bescheidService.getBescheidDocumentFile).toHaveBeenCalled();
    });
  });

  describe('getBescheid', () => {
    it('should not call bescheidService.loadBescheidDocumentByUri', (done) => {
      component.ngOnChanges(changes);

      component.bescheidStateResource$.subscribe(() => {
        expect(bescheidService.loadBescheidDocumentByUri).not.toHaveBeenCalled();
        done();
      });
    });

    it('should call bescheidService.loadBescheidDocumentByUri', (done) => {
      const bescheidResource: BescheidResource = createBescheidResource([
        BescheidLinkRel.BESCHEID_DOCUMENT,
      ]);
      const bescheidStateResource: StateResource<BescheidResource> =
        createStateResource(bescheidResource);
      bescheidService.getBescheidDraft.mockReturnValue(of(bescheidStateResource));

      component.ngOnChanges(changes);

      component.bescheidStateResource$.subscribe(() => {
        expect(bescheidService.loadBescheidDocumentByUri).toHaveBeenCalled();
        done();
      });
    });
  });
});
