import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import faker from '@faker-js/faker';
import { Observable, of } from 'rxjs';
import { singleColdCompleted } from '../../../tech-shared/test/marbles';
import {
  createOrganisationsEinheitListResource,
  createOrganisationsEinheitResource,
} from '../../test/organisations-einheit';
import { OrganisationsEinheitResourceSearchService } from './organisations-einheit-resource-search.service';
import {
  OrganisationsEinheitListResource,
  OrganisationsEinheitResource,
} from './organisations-einheit.model';
import { OrganisationsEinheitService } from './organisations-einheit.service';

jest.mock('./organisations-einheit-resource-search.service');

describe('OrganisationsEinheitService', () => {
  let service: OrganisationsEinheitService;

  let searchService: Mock<OrganisationsEinheitResourceSearchService>;

  const listResource: OrganisationsEinheitListResource = createOrganisationsEinheitListResource();
  const listStateResource: StateResource<OrganisationsEinheitListResource> =
    createStateResource(listResource);

  beforeEach(() => {
    searchService = mock(OrganisationsEinheitResourceSearchService);

    service = new OrganisationsEinheitService(useFromMock(searchService));
  });

  describe('get search result list', () => {
    it('should call search service', () => {
      service.getSearchResultList();

      expect(searchService.getResultList).toHaveBeenCalled();
    });

    it('should return result', (done) => {
      searchService.getResultList.mockReturnValue(of(listStateResource));

      service
        .getSearchResultList()
        .subscribe((result: StateResource<OrganisationsEinheitListResource>) => {
          expect(result).toBe(listStateResource);
          done();
        });
    });
  });

  describe('search', () => {
    const searchBy: string = faker.random.word();

    it('should call search service with search string', () => {
      service.search(searchBy);

      expect(searchService.search).toHaveBeenCalledWith(searchBy);
    });
  });

  describe('clear search result', () => {
    it('should call search service', () => {
      service.clearSearchResult();

      expect(searchService.clearResultList).toHaveBeenCalledWith();
    });
  });

  describe('get selected result', () => {
    const organisationsEinheitResource: OrganisationsEinheitResource =
      createOrganisationsEinheitResource();

    beforeEach(() => {
      searchService.getSelectedResult.mockReturnValue(of(organisationsEinheitResource));
    });

    it('should call service', () => {
      service.getSelectedResult();

      expect(searchService.getSelectedResult).toHaveBeenCalled();
    });

    it('should return result', () => {
      const selectedResult$: Observable<OrganisationsEinheitResource> = service.getSelectedResult();

      expect(selectedResult$).toBeObservable(singleColdCompleted(organisationsEinheitResource));
    });
  });

  describe('select search result', () => {
    const organisationsEinheitResource: OrganisationsEinheitResource =
      createOrganisationsEinheitResource();

    it('should call service', () => {
      service.selectSearchResult(organisationsEinheitResource);

      expect(searchService.selectResult).toHaveBeenCalledWith(organisationsEinheitResource);
    });
  });

  describe('clear selected result', () => {
    it('should call service', () => {
      service.clearSelectedResult();

      expect(searchService.clearSelectedResult).toHaveBeenCalled();
    });
  });
});
