import { OrganisationsEinheitResource } from '@alfa-client/collaboration-shared';
import { Component, EventEmitter, Input, Output } from '@angular/core';
import { InstantSearchResult } from '@ods/system';
import { SearchOrganisationsEinheitFormService } from '../search-organisations-einheit.formservice';

@Component({
  selector: 'alfa-search-organisations-einheit-form',
  templateUrl: './search-organisations-einheit-form.component.html',
  providers: [SearchOrganisationsEinheitFormService],
})
export class SearchOrganisationsEinheitFormComponent {
  @Input() set organisationsEinheiten(organisationsEinheiten: OrganisationsEinheitResource[]) {
    this.mapOrganisationsEinheitResources(organisationsEinheiten);
  }

  @Output() public search: EventEmitter<string> = new EventEmitter();
  @Output() public selectSearchResult: EventEmitter<OrganisationsEinheitResource> =
    new EventEmitter();
  @Output() public clearSearchResult: EventEmitter<string> = new EventEmitter();

  public searchResults: InstantSearchResult<OrganisationsEinheitResource>[];

  constructor(public formService: SearchOrganisationsEinheitFormService) {}

  mapOrganisationsEinheitResources(organisationsEinheiten: OrganisationsEinheitResource[]): void {
    this.searchResults = organisationsEinheiten.map(
      (organisationsEinheiten: OrganisationsEinheitResource) =>
        this.mapToInstantantSearchResult(organisationsEinheiten),
    );
  }

  mapToInstantantSearchResult(
    organisationsEinheit: OrganisationsEinheitResource,
  ): InstantSearchResult<OrganisationsEinheitResource> {
    return <any>{
      title: organisationsEinheit.name,
      description: `${organisationsEinheit.anschrift.strasse} ${organisationsEinheit.anschrift.hausnummer}, ${organisationsEinheit.anschrift.plz} ${organisationsEinheit.anschrift.ort}`,
      data: organisationsEinheit,
    };
  }
}
