import {
  argsToTemplate,
  componentWrapperDecorator,
  moduleMetadata,
  type Meta,
  type StoryObj,
} from '@storybook/angular';

import { SaveIconComponent } from '../../icons/save-icon/save-icon.component';
import { UserIconComponent } from '../../icons/user-icon/user-icon.component';
import { DropdownMenuButtonComponent } from '../dropdown-menu-button/dropdown-menu-button.component';
import { DropdownMenuComponent } from './dropdown-menu.component';

const meta: Meta<DropdownMenuComponent> = {
  title: 'Dropdown menu/Dropdown menu',
  component: DropdownMenuComponent,
  decorators: [
    moduleMetadata({
      imports: [
        DropdownMenuComponent,
        DropdownMenuButtonComponent,
        SaveIconComponent,
        UserIconComponent,
      ],
    }),
    componentWrapperDecorator((story) => `<div class="flex justify-center mb-32">${story}</div>`),
  ],
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
};

export default meta;
type Story = StoryObj<DropdownMenuComponent>;

export const Default: Story = {
  args: { alignTo: 'left', label: '', buttonClass: '' },
  argTypes: {
    alignTo: {
      control: 'select',
      options: ['left', 'right'],
      table: {
        defaultValue: { summary: 'left' },
      },
    },
    buttonClass: { description: 'Tailwind class for button' },
    label: { description: 'Aria-label for button' },
  },
  render: (args) => ({
    props: args,
    template: `<ods-dropdown-menu ${argsToTemplate(args)}>
        <ods-user-icon button-content />
        <ods-dropdown-menu-button caption="Lorem" />
        <ods-dropdown-menu-button caption="Ipsum" />
        <ods-dropdown-menu-button caption="Dolor" />
      </ods-dropdown-menu>`,
  }),
};

export const LongText: Story = {
  render: (args) => ({
    props: args,
    template: `<ods-dropdown-menu ${argsToTemplate(args)}>
        <p button-content>Trigger popup</p>
        <ods-dropdown-menu-button caption="Lorem" />
        <ods-dropdown-menu-button caption="Lorem ipsum dolor sit amet" />
      </ods-dropdown-menu>`,
  }),
};

export const ItemsWithIcons: Story = {
  render: (args) => ({
    props: args,
    template: `<ods-dropdown-menu ${argsToTemplate(args)}>
        <p button-content>Trigger popup</p>
        <ods-dropdown-menu-button caption="Lorem">
          <ods-save-icon icon size="small" />
        </ods-dropdown-menu-button>
        <ods-dropdown-menu-button caption="Lorem ipsum dolor sit amet">
          <ods-save-icon icon size="small" />
        </ods-dropdown-menu-button>
      </ods-dropdown-menu>`,
  }),
};
