import { convertForDataTest } from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import faker from '@faker-js/faker';
import { getDataTestIdOf } from '../../../../../tech-shared/test/data-test';
import { TextareaComponent } from './textarea.component';

describe('TextareaComponent', () => {
  let component: TextareaComponent;
  let fixture: ComponentFixture<TextareaComponent>;
  const label: string = faker.word.noun();
  const dataTestId: string = `${convertForDataTest(label)}-textarea`;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [ReactiveFormsModule, TextareaComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(TextareaComponent);
    component = fixture.componentInstance;
    component.label = label;
    component.fieldControl = new FormControl();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('aria', () => {
    it('should have aria-invalid attribute', () => {
      component.variant = 'error';

      fixture.detectChanges();

      const element: HTMLElement = getElementFromFixture(fixture, getDataTestIdOf(dataTestId));
      expect(element.hasAttribute('aria-invalid')).toBe(true);
    });

    it('should have aria-invalid value false', () => {
      component.variant = 'default';

      fixture.detectChanges();

      const element: HTMLElement = getElementFromFixture(fixture, getDataTestIdOf(dataTestId));
      expect(element.getAttribute('aria-invalid')).toEqual('false');
    });

    it('should have aria-invalid value true', () => {
      component.variant = 'error';

      fixture.detectChanges();

      const element: HTMLElement = getElementFromFixture(fixture, getDataTestIdOf(dataTestId));
      expect(element.getAttribute('aria-invalid')).toEqual('true');
    });
  });

  describe('focus', () => {
    beforeEach(() => {
      component.textAreaElement.nativeElement.focus = jest.fn();
    });

    it('should focus', () => {
      component.focus = true;

      expect(component.textAreaElement.nativeElement.focus).toHaveBeenCalled();
    });

    it('should not focus', () => {
      component.focus = false;

      expect(component.textAreaElement.nativeElement.focus).not.toHaveBeenCalled();
    });
  });
});
