/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createEmptyStateResource, HasLinkPipe } from '@alfa-client/tech-shared';
import { SpinnerComponent } from '@alfa-client/ui';
import { VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { VorgangForwardFormComponent } from './vorgang-forward-form/vorgang-forward-form.component';
import { VorgangForwardFormularComponent } from './vorgang-forward-formular.component';

describe('VorgangForwardFormularComponent', () => {
  let component: VorgangForwardFormularComponent;
  let fixture: ComponentFixture<VorgangForwardFormularComponent>;

  const forwardFormular: string = '[data-test-id="forward-form"]';

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangForwardFormularComponent,
        HasLinkPipe,
        MockComponent(SpinnerComponent),
        MockComponent(VorgangForwardFormComponent),
      ],
    });
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(VorgangForwardFormularComponent);
    component = fixture.componentInstance;
    component.pendingForwardCommand = createEmptyStateResource();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('forward formular', () => {
    it('should show', () => {
      component.vorgang = createVorgangWithEingangResource([VorgangWithEingangLinkRel.FORWARD]);
      fixture.detectChanges();

      const element = fixture.nativeElement.querySelector(forwardFormular);

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should not show', () => {
      component.vorgang = createVorgangWithEingangResource();
      fixture.detectChanges();

      const element = fixture.nativeElement.querySelector(forwardFormular);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });
});
