import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { createHintListResource } from 'libs/hint-shared/test/hint';
import { createEmptyListResource } from 'libs/tech-shared/test/resource';
import { HintListResource } from '../hint.model';
import { HintPartialState, initialState } from './hint.reducer';

import * as HintUtil from '../hint.util';
import * as HintSelectors from './hint.selectors';

describe('Hint Selectors', () => {
  let state: HintPartialState;

  const hintList: StateResource<HintListResource> = createStateResource(createHintListResource());

  beforeEach(() => {
    state = {
      HintState: {
        ...initialState,
        hintList,
      },
    };
  });

  describe('hasNoOrganisationsEinheitId', () => {
    const getHasUserNoOrganisationsEinheitIdSpy: jest.SpyInstance<
      boolean,
      [hintList: HintListResource]
    > = jest.spyOn(HintUtil, 'hasUserNoOrganisationsEinheitIdHint');

    it('should return stateResource with hint if exist', () => {
      getHasUserNoOrganisationsEinheitIdSpy.mockReturnValue(true);

      const result: boolean = HintSelectors.hasUserNoOrganisationsEinheitIdHint.projector(
        state.HintState,
      );

      expect(result).toBeTruthy();
    });

    it('should return emptyStateResource if not exist', () => {
      getHasUserNoOrganisationsEinheitIdSpy.mockReturnValue(false);

      const result: boolean = HintSelectors.hasUserNoOrganisationsEinheitIdHint.projector(
        state.HintState,
      );

      expect(result).toBeFalsy();
    });

    it('should return emptyStateResource if resource is null', () => {
      const result: boolean = HintSelectors.hasUserNoOrganisationsEinheitIdHint.projector({
        ...state.HintState,
        hintList: createEmptyListResource(),
      });

      expect(result).toBeFalsy();
    });
  });
});
