/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandOrder } from '@alfa-client/command-shared';
import { ListResource, StringBasedKeyMap } from '@alfa-client/tech-shared';
import { Resource } from '@ngxp/rest';

export interface Historie {
  createdAt: Date;
  createdByName: string;
  status: any;
  finishedAt: Date;
  errorMessage: string;
  order: any;
  body: object;
}

export interface HistorieResource extends Historie, Resource {}

export interface HistorieListResource extends ListResource {}

export enum CommandOrderType {
  USER = 'User',
  FORWARDING = 'Forwarding',
  KOMMENTAR = 'Kommentar',
  POSTFACH_NACHRICHT = 'PostfachNachricht',
  VORGANG = 'Vorgang',
  WIEDERVORLAGE = 'Wiedervorlage',
  LOESCH_ANFORDERUNG = 'LoeschAnforderung',
  SET_AKTENZEICHEN = 'SetAktenzeichen',
  BESCHEID = 'Bescheid',
}

export const ORDER_TYPE_BY_COMMAND_ORDER: StringBasedKeyMap<CommandOrderType> = {
  [CommandOrder.ASSIGN_USER]: CommandOrderType.USER,
  [CommandOrder.CREATE_KOMMENTAR]: CommandOrderType.KOMMENTAR,
  [CommandOrder.EDIT_KOMMENTAR]: CommandOrderType.KOMMENTAR,
  [CommandOrder.CREATE_WIEDERVORLAGE]: CommandOrderType.WIEDERVORLAGE,
  [CommandOrder.EDIT_WIEDERVORLAGE]: CommandOrderType.WIEDERVORLAGE,
  [CommandOrder.FORWARD_FAILED]: CommandOrderType.FORWARDING,
  [CommandOrder.REDIRECT_VORGANG]: CommandOrderType.FORWARDING,
  [CommandOrder.FORWARD_SUCCESSFULL]: CommandOrderType.FORWARDING,
  [CommandOrder.SEND_POSTFACH_NACHRICHT]: CommandOrderType.POSTFACH_NACHRICHT,
  [CommandOrder.RECEIVE_POSTFACH_NACHRICHT]: CommandOrderType.POSTFACH_NACHRICHT,
  [CommandOrder.RESEND_POSTFACH_NACHRICHT]: CommandOrderType.POSTFACH_NACHRICHT,
  [CommandOrder.WIEDERVORLAGE_ERLEDIGEN]: CommandOrderType.WIEDERVORLAGE,
  [CommandOrder.WIEDERVORLAGE_WIEDEREROEFFNEN]: CommandOrderType.WIEDERVORLAGE,
  [CommandOrder.VORGANG_ANNEHMEN]: CommandOrderType.VORGANG,
  [CommandOrder.VORGANG_VERWERFEN]: CommandOrderType.VORGANG,
  [CommandOrder.VORGANG_ZURUECKHOLEN]: CommandOrderType.VORGANG,
  [CommandOrder.VORGANG_BEARBEITEN]: CommandOrderType.VORGANG,
  [CommandOrder.VORGANG_BESCHEIDEN]: CommandOrderType.VORGANG,
  [CommandOrder.VORGANG_ZURUECKSTELLEN]: CommandOrderType.VORGANG,
  [CommandOrder.VORGANG_ABSCHLIESSEN]: CommandOrderType.VORGANG,
  [CommandOrder.VORGANG_WIEDEREROEFFNEN]: CommandOrderType.VORGANG,
  [CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN]: CommandOrderType.LOESCH_ANFORDERUNG,
  [CommandOrder.VORGANG_LOESCHEN]: CommandOrderType.LOESCH_ANFORDERUNG,
  [CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN]: CommandOrderType.LOESCH_ANFORDERUNG,
  [CommandOrder.SET_AKTENZEICHEN]: CommandOrderType.SET_AKTENZEICHEN,
  [CommandOrder.SEND_BESCHEID]: CommandOrderType.BESCHEID,
  [CommandOrder.UPDATE_BESCHEID]: CommandOrderType.BESCHEID,
};
