import { ResourceRedirectComponent } from '@alfa-client/resource-redirect';
import { ResourceRedirectService } from '@alfa-client/resource-redirect-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ActivatedRoute } from '@angular/router';
import { Observable, of } from 'rxjs';

describe('ResourceRedirectComponent', () => {
  let component: ResourceRedirectComponent;
  let fixture: ComponentFixture<ResourceRedirectComponent>;
  let resourceRedirectService: Mock<ResourceRedirectService>;

  const initActivatedRoute = (queryParamKey?: string): { queryParamMap: Observable<Map<string, string>> } => ({
    queryParamMap: of(new Map(queryParamKey ? [[queryParamKey, 'some-uri']] : [])),
  });

  const createComponentWithQueryParam = (queryParamKey?: string): void => {
    TestBed.configureTestingModule({
      imports: [ResourceRedirectComponent],
      providers: [
        { provide: ActivatedRoute, useValue: initActivatedRoute(queryParamKey) },
        { provide: ResourceRedirectService, useValue: resourceRedirectService },
      ],
    });

    fixture = TestBed.createComponent(ResourceRedirectComponent);
    component = fixture.componentInstance;
  };

  beforeEach(() => {
    TestBed.resetTestingModule();

    resourceRedirectService = mock(ResourceRedirectService);
  });

  it('should create', () => {
    createComponentWithQueryParam();

    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call resource service redirectToResource', () => {
      createComponentWithQueryParam('uri');

      component.ngOnInit();

      expect(resourceRedirectService.redirectToResource).toHaveBeenCalledWith('some-uri');
    });

    it('should not call resource service redirectToResource with other query param', () => {
      createComponentWithQueryParam('some-other-param');

      component.ngOnInit();

      expect(resourceRedirectService.redirectToResource).not.toHaveBeenCalled();
    });

    it('should not call resource service redirectToResource without uri param', () => {
      createComponentWithQueryParam();

      component.ngOnInit();

      expect(resourceRedirectService.redirectToResource).not.toHaveBeenCalled();
    });
  });
});
