import { MonoTypeOperatorFunction, Observable, tap } from 'rxjs';

export function debugTap<T>(fn: (value: T) => void) {
  return function (source: Observable<T>) {
    source
      .pipe(
        debug(),
        tap((value) => fn(value)),
      )
      .subscribe();
    return source;
  };
}

function debug<T>(): MonoTypeOperatorFunction<T> {
  return tap({
    next(value) {
      console.log(
        `%c[Alfa Client: Next]`,
        'background: #009688; color: #fff; padding: 3px; font-size: 9px;',
        value,
      );
    },
    error(error) {
      console.log(
        `%[Alfa Client: Error]`,
        'background: #E91E63; color: #fff; padding: 3px; font-size: 9px;',
        error,
      );
    },
    complete() {
      console.log(
        `%c[Alfa Client]: Complete`,
        'background: #00BCD4; color: #fff; padding: 3px; font-size: 9px;',
      );
    },
  });
}
