import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule } from '@angular/forms';
import { MatIcon } from '@angular/material/icon';
import { MatFormFieldModule } from '@angular/material/form-field';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { ConvertForDataTestPipe } from '@alfa-client/tech-shared';
import { MockComponent } from 'ng-mocks';
import { IconButtonWithSpinnerComponent } from '../../icon-button-with-spinner/icon-button-with-spinner.component';
import { CheckboxEnumEditorComponent } from './checkbox-enum-editor.component';
import { UiModule } from '../../ui.module';

describe('CheckboxEnumEditorComponent', () => {
  let component: CheckboxEnumEditorComponent;
  let fixture: ComponentFixture<CheckboxEnumEditorComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        CheckboxEnumEditorComponent,
        MatIcon,
        ConvertForDataTestPipe,
        MockComponent(IconButtonWithSpinnerComponent),
      ],
      imports: [UiModule, MatFormFieldModule, ReactiveFormsModule, BrowserAnimationsModule],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(CheckboxEnumEditorComponent);
    component = fixture.componentInstance;
    component.defaultItem = { type: 'enumType' };
    component.checkedItem = { type: 'positive' };
    component.uncheckedItem = { type: 'negative' };
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('setSelection', () => {
    const writeMethodMock = jest.fn();

    beforeEach(() => {
      component.writeValue = writeMethodMock;
    });

    it('should write checked value on selected', () => {
      component.setSelection({ source: null, checked: true });

      expect(writeMethodMock).toHaveBeenCalledWith('positive');
    });

    it('should write unchecked value on selected', () => {
      component.setSelection({ source: null, checked: false });

      expect(writeMethodMock).toHaveBeenCalledWith('negative');
    });
  });
});
