/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import {
  AbstractControl,
  FormGroupDirective,
  ReactiveFormsModule,
  UntypedFormBuilder,
} from '@angular/forms';
import { MatNativeDateModule } from '@angular/material/core';
import { MatDatepickerModule } from '@angular/material/datepicker';
import { MatIconModule } from '@angular/material/icon';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { ConvertForDataTestPipe, createEmptyStateResource } from '@alfa-client/tech-shared';
import { MockComponent } from 'ng-mocks';
import { SpinnerComponent } from '../../spinner/spinner.component';
import { ValidationErrorComponent } from '../../validation-error/validation-error.component';
import { FileUploadEditorComponent } from './file-upload-editor.component';

describe('FileUploadEditorComponent', () => {
  let component: FileUploadEditorComponent;
  let fixture: ComponentFixture<FileUploadEditorComponent>;
  const fb = new UntypedFormBuilder();
  const formGroupDirective = new FormGroupDirective([], []);
  formGroupDirective.form = fb.group({
    attachments: fb.control(null),
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        FileUploadEditorComponent,
        ConvertForDataTestPipe,
        MockComponent(ValidationErrorComponent),
        MockComponent(SpinnerComponent),
      ],
      imports: [
        MatFormFieldModule,
        MatInputModule,
        MatIconModule,
        MatDatepickerModule,
        MatNativeDateModule,
        ReactiveFormsModule,
        BrowserAnimationsModule,
      ],
      providers: [
        FormGroupDirective,
        UntypedFormBuilder,
        {
          provide: FormGroupDirective,
          useValue: formGroupDirective,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(FileUploadEditorComponent);
    component = fixture.componentInstance;
    component.parentFormArrayName = 'attachments';
    component.uploadInProgress = createEmptyStateResource();
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('after OnInit', () => {
    it('should get fileLinkControls', () => {
      expect(component.fileLinkControls).toBeInstanceOf(AbstractControl);
    });
  });

  describe('data test id', () => {
    it('should exist for input', () => {
      component.label = 'Ein Label';
      fixture.detectChanges();

      const element: HTMLElement = fixture.nativeElement.querySelector(
        '[data-test-id="Ein_Label-file-upload-input"]',
      );

      expect(element).toBeInstanceOf(HTMLElement);
    });
  });

  describe('click at file input', () => {
    it('should call resetInput()', () => {
      component.resetInput = jest.fn();
      const fileInput = fixture.nativeElement.querySelector('[data-test-id="-file-upload-input"]');

      fileInput.click();
      fixture.detectChanges();

      expect(component.resetInput).toHaveBeenCalled();
    });
  });
});
