import { FormBuilder, FormControl } from '@angular/forms';
import { mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangService } from '@alfa-client/vorgang-shared';
import { EMPTY_STRING } from '@alfa-client/tech-shared';
import { createVorgangWithEingangResource } from '../../../../vorgang-shared/test/vorgang';
import { EMPTY } from 'rxjs';
import { AktenzeichenEditDialogFormservice } from './aktenzeichen-edit-dialog.formservice';

const vorgangService = mock(VorgangService);
const service = new AktenzeichenEditDialogFormservice(
  new FormBuilder(),
  useFromMock(vorgangService),
);

describe('getValue', () => {
  const formControl = new FormControl('aktenzeichen');
  jest.spyOn(service, 'getAktenzeichenFormControl').mockReturnValue(formControl);

  it('should return form control value', () => {
    const value = service.getValue();

    expect(value).toEqual('aktenzeichen');
  });

  it('should return trimmed value', () => {
    formControl.setValue('  aktenzeichen  ');

    const value = service.getValue();

    expect(value).toEqual('aktenzeichen');
  });

  it.each([null, EMPTY_STRING])('should return null', (aktenzeichen: string) => {
    formControl.setValue(aktenzeichen);

    const value = service.getValue();

    expect(value).toBeNull();
  });
});

describe('doSubmit', () => {
  it('should set aktenzeichen', () => {
    const vorgang = createVorgangWithEingangResource();
    service.setVorgangWithEingangResource(vorgang);
    jest.spyOn(service, 'getValue').mockReturnValue('aktenzeichen');
    vorgangService.setAktenzeichen.mockReturnValue(EMPTY);

    service.submit();

    expect(vorgangService.setAktenzeichen).toBeCalledWith(vorgang, 'aktenzeichen');
  });
});
