import { BescheidService } from '@alfa-client/bescheid-shared';
import { CommandResource } from '@alfa-client/command-shared';
import {
  StateResource,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { Mock, mock } from '@alfa-client/test-utils';
import { OzgcloudDialogService, OzgcloudStrokedButtonWithSpinnerComponent } from '@alfa-client/ui';
import { VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { DialogRef } from '@angular/cdk/dialog';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { cold } from 'jest-marbles';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { CommandLinkRel } from '../../../../../../../../command-shared/src/lib/command.linkrel';
import { createCommandResource } from '../../../../../../../../command-shared/test/command';
import { createVorgangWithEingangResource } from '../../../../../../../../vorgang-shared/test/vorgang';
import { VorgangDetailBescheidenUeberspringenDialogComponent } from './vorgang-detail-bescheiden-ueberspringen-dialog.component';

describe('VorgangDetailBescheidenUeberspringenButtonComponent', () => {
  let component: VorgangDetailBescheidenUeberspringenDialogComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenUeberspringenDialogComponent>;

  let dialogRef: Mock<DialogRef>;
  let bescheidService: Mock<BescheidService>;
  let ozgcloudDialogService: Mock<OzgcloudDialogService>;
  let vorgangService: Mock<VorgangService>;

  beforeEach(async () => {
    dialogRef = mock(DialogRef);
    bescheidService = mock(BescheidService);
    ozgcloudDialogService = mock(OzgcloudDialogService);
    vorgangService = mock(VorgangService);

    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailBescheidenUeberspringenDialogComponent,
        MockComponent(OzgcloudStrokedButtonWithSpinnerComponent),
        MockComponent(MatIcon),
      ],
      providers: [
        {
          provide: DialogRef,
          useValue: dialogRef,
        },
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
        {
          provide: OzgcloudDialogService,
          useValue: ozgcloudDialogService,
        },
        {
          provide: VorgangService,
          useValue: vorgangService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenUeberspringenDialogComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('onCancel', () => {
    it('should close dialog', () => {
      component.onCancel();

      expect(dialogRef.close).toHaveBeenCalled();
    });
  });

  describe('onConfirm', () => {
    let vorgangWithEingangResource: VorgangWithEingangResource;
    const successfullyDoneCommandStateResource: StateResource<CommandResource> =
      createStateResource(createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]));

    beforeEach(() => {
      vorgangWithEingangResource = createVorgangWithEingangResource();
      vorgangService.getVorgangWithEingang.mockReturnValue(
        of(createStateResource(vorgangWithEingangResource)),
      );
      bescheidService.bescheidErstellungUeberspringen.mockReturnValue(
        of(successfullyDoneCommandStateResource),
      );
    });

    it('should get vorgang with eingang', () => {
      component.onConfirm();

      expect(vorgangService.getVorgangWithEingang).toHaveBeenCalled();
    });

    it('should call bescheid service erstellung ueberspringen', (done) => {
      component.onConfirm();

      component.bescheiderstellungUeberspringen$.subscribe(() => {
        expect(bescheidService.bescheidErstellungUeberspringen).toHaveBeenCalledWith(
          vorgangWithEingangResource,
        );
        done();
      });
    });

    it('should close all dialogs on success', (done) => {
      component.onConfirm();

      component.bescheiderstellungUeberspringen$.subscribe(() => {
        expect(ozgcloudDialogService.closeAll).toHaveBeenCalled();
        done();
      });
    });

    it('should not close all dialogs on command not done successfully', (done) => {
      bescheidService.bescheidErstellungUeberspringen.mockReturnValue(
        of(createEmptyStateResource()),
      );

      component.onConfirm();

      component.bescheiderstellungUeberspringen$.subscribe(() => {
        expect(ozgcloudDialogService.closeAll).not.toHaveBeenCalled();
        done();
      });
    });

    it('should return command', () => {
      component.onConfirm();

      expect(component.bescheiderstellungUeberspringen$).toBeObservable(
        cold('(a|)', { a: successfullyDoneCommandStateResource }),
      );
    });
  });

  describe('onClose', () => {
    it('should close dialog', () => {
      component.onClose();

      expect(dialogRef.close).toHaveBeenCalled();
    });
  });
});
