/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AfterViewInit, Component, ElementRef, Input, OnChanges } from '@angular/core';
import {
  createEmptyStateResource,
  createStateResource,
  StateResource,
} from '@alfa-client/tech-shared';
import { VorgangResource } from '@alfa-client/vorgang-shared';
import {
  WiedervorlageLinkRel,
  WiedervorlageListLinkRel,
  WiedervorlageResource,
  WiedervorlageService,
} from '@alfa-client/wiedervorlage-shared';
import { getEmbeddedResource, hasLink } from '@ngxp/rest';
import { isNil, isNull } from 'lodash-es';
import { Observable } from 'rxjs';
import { filter, map, startWith } from 'rxjs/operators';

@Component({
  selector: 'alfa-wiedervorlage-list-in-vorgang-list-container',
  templateUrl: './wiedervorlage-list-in-vorgang-list-container.component.html',
  styleUrls: ['./wiedervorlage-list-in-vorgang-list-container.component.scss'],
})
export class WiedervorlageListInVorgangListContainerComponent implements OnChanges, AfterViewInit {
  @Input() vorgang: VorgangResource;

  wiedervorlagenStateResource$: Observable<StateResource<WiedervorlageResource[]>>;

  public readonly wiedervorlageListLinkRel = WiedervorlageListLinkRel;

  constructor(
    private wiedervorlageService: WiedervorlageService,
    private elementRef: ElementRef,
  ) {}

  ngOnChanges(): void {
    if (!isNil(this.vorgang)) {
      this.loadWiedervorlagen();
    }
  }

  ngAfterViewInit(): void {
    if (this.elementRef.nativeElement.getBoundingClientRect().bottom >= window.innerHeight) {
      this.elementRef.nativeElement.classList.add('up');
    }
  }

  loadWiedervorlagen(): void {
    this.wiedervorlagenStateResource$ = this.wiedervorlageService
      .loadWiedervorlagenByVorgangStrict(this.vorgang)
      .pipe(
        filter((listStateResource) => !isNull(listStateResource.resource)),
        map((listStateResource) =>
          getEmbeddedResource<WiedervorlageResource[]>(
            listStateResource.resource,
            this.wiedervorlageListLinkRel.WIEDERVORLAGE_LIST,
          ),
        ),
        map((wiedervorlagen) =>
          createStateResource(
            wiedervorlagen ?
              wiedervorlagen.filter((wiedervorlage) =>
                hasLink(wiedervorlage, WiedervorlageLinkRel.ERLEDIGEN),
              )
            : [],
          ),
        ),
        startWith(createEmptyStateResource<WiedervorlageResource[]>()),
      );
  }
}
