package de.ozgcloud.alfa.collaboration;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.stream.Stream;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.ModelBuilder;
import de.ozgcloud.alfa.common.command.CommandController;
import de.ozgcloud.alfa.vorgang.VorgangController;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor
class CollaborationModelAssembler implements RepresentationModelAssembler<Collaboration, EntityModel<Collaboration>> {

	static final LinkRelation REL_CREATE_COLLABORATION_REQUEST = LinkRelation.of("createCollaborationRequest");

	private final VorgangController vorgangController;

	@Override
	public EntityModel<Collaboration> toModel(Collaboration collaboration) {
		return ModelBuilder.fromEntity(collaboration)
				// TODO: Wenn Schnittstelle zum laden der Collaboration existiert, muss self
				// link ergänzt werden
				.buildModel();
	}

	public CollectionModel<EntityModel<Collaboration>> toCollectionModel(Stream<? extends Collaboration> entities, String vorgangId) {
		var collectionModel = CollectionModel.of(entities.map(this::toModel).toList())
				.add(linkTo(methodOn(CollaborationController.class).getAllByVorgangId(vorgangId)).withSelfRel());
		return collectionModel
				.addIf(collectionModel.getContent().isEmpty(), () -> buildCreateCollaborationRequestLink(vorgangId));
	}

	Link buildCreateCollaborationRequestLink(String vorgangId) {
		var vorgang = vorgangController.getVorgang(vorgangId);
		return linkTo(methodOn(CommandController.CommandByRelationController.class).createCommand(vorgang.getId(), vorgang.getId(),
				vorgang.getVersion(), null)).withRel(REL_CREATE_COLLABORATION_REQUEST);
	}

}
