/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common;

import java.nio.charset.StandardCharsets;
import java.util.Collection;
import java.util.Optional;
import java.util.stream.StreamSupport;

import io.grpc.Metadata;
import io.grpc.Metadata.Key;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class GrpcUtil {

	public static final String VORGANG_MANAGER_GRPC_CLIENT = "vorgang-manager";

	public static final String ZUFI_MANAGER_GRPC_CLIENT = "zufi-manager";

	public static final String SERVICE_KEY = "GRPC_SERVICE";

	public static Key<String> keyOfString(String key) {
		return Key.of(key, Metadata.ASCII_STRING_MARSHALLER);
	}

	public static Key<byte[]> createKeyOf(String key) {
		return Key.of(key, Metadata.BINARY_BYTE_MARSHALLER);
	}

	public static String getFromHeaders(String key, Metadata headers) {
		return Optional.ofNullable(headers.get(createKeyOf(key)))
				.map(GrpcUtil::byteToString)
				.orElse(null);
	}

	public static Collection<String> getCollection(String key, Metadata headers) {
		return StreamSupport.stream(headers.getAll(createKeyOf(key)).spliterator(), false)
				.map(GrpcUtil::byteToString)
				.toList();
	}

	private static String byteToString(byte[] bytes) {
		return new String(bytes, StandardCharsets.UTF_8);
	}
}