package de.ozgcloud.alfa.historie;

import java.util.List;
import java.util.stream.Stream;

import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.user.UserService;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class VorgangChangeHistoryService {

	private final UserService userService;
	private final CommandService commandService;

	public VorgangChangeHistory createVorgangChangeHistory(VorgangWithEingang vorgang) {
		var commands = commandService.findFinishedCommands(vorgang.getId()).toList();
		return VorgangChangeHistory.builder()
				.statusChangeHistory(createStatusChangeHistory(vorgang, commands).toList())
				.aktenzeichenChangeHistory(createAktenzeichenChangeHistory(vorgang, commands).toList())
				.assignedUserChangeHistory(createAssignedUserChangeHistory(vorgang, commands).toList())
				.build();
	}

	Stream<VorgangChange> createStatusChangeHistory(VorgangWithEingang vorgang, List<Command> commands) {
		return StatusChangeHistoryBuilder.builder()
				.withCommands(commands)
				.withOrganisationseinheitenID(vorgang.getOrganisationseinheitenID())
				.build();
	}

	Stream<VorgangChange> createAktenzeichenChangeHistory(VorgangWithEingang vorgang, List<Command> commands) {
		return AktenzeichenChangeHistoryBuilder.builder()
				.withCommands(commands)
				.withOrganisationseinheitenID(vorgang.getOrganisationseinheitenID())
				.build();
	}

	Stream<VorgangChange> createAssignedUserChangeHistory(VorgangWithEingang vorgang, List<Command> commands) {
		return AssignedUserChangeHistoryBuilder.builder()
				.withCommands(commands)
				.withOrganisationseinheitenID(vorgang.getOrganisationseinheitenID())
				.withUserProfileCache(UserProfileCache.create(userService::getById))
				.build();
	}

}
