/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.IOException;

import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

class RequestIdFilterTest {

	@InjectMocks
	private RequestIdFilter filter;
	@Mock
	private FilterChain filterChain;
	@Mock
	private HttpServletRequest request;
	@Mock
	private HttpServletResponse response;
	@Mock
	private CallScope callScope;

	@Spy
	private RequestAttributes attributes;

	@Test
	void shouldContinueFilterChain() throws IOException, ServletException {
		callDoFilterInternal();

		verify(filterChain).doFilter(request, response);
	}

	@Test
	void shouldStartCallScope() {
		callDoFilterInternal();

		verify(callScope).startScope();
	}

	@Test
	void shouldEndCallScopeAfterException() throws IOException, ServletException {
		doThrow(RuntimeException.class).when(filterChain).doFilter(any(), any());

		assertThrows(RuntimeException.class, this::callDoFilterInternal);

		verify(callScope).endScope();
	}

	private void callDoFilterInternal() {
		try {
			filter.doFilterInternal(request, response, filterChain);
		} catch (ServletException | IOException e) {
			throw new RuntimeException(e);
		}
	}
}
