package de.ozgcloud.alfa.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatcher;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.document.DocumentServiceGrpc.DocumentServiceBlockingStub;
import de.ozgcloud.document.GrpcGetDocumentRequest;

class DocumentRemoteServiceTest {

	@Mock
	private DocumentServiceBlockingStub documentServiceStub;
	@Mock
	private DocumentMapper documentMapper;
	@Spy
	@InjectMocks
	private DocumentRemoteService service;

	@Nested
	class TestGetDocument {

		private static final ArgumentMatcher<GrpcGetDocumentRequest> REQUEST_MATCHER = request -> request.getId().equals(DocumentTestFactory.ID);

		private final Document document = DocumentTestFactory.create();

		@BeforeEach
		void init() {
			when(documentServiceStub.getDocument(argThat(REQUEST_MATCHER))).thenReturn(GrpcGetDocumentResponseTestFactory.create());
			when(documentMapper.fromGrpc(GrpcGetDocumentResponseTestFactory.GRPC_DOCUMENT)).thenReturn(document);
		}

		@Test
		void shouldCallGrpcService() {
			callService();

			verify(documentServiceStub).getDocument(argThat(REQUEST_MATCHER));
		}

		@Test
		void shouldCallMapper() {
			callService();

			verify(documentMapper).fromGrpc(GrpcGetDocumentResponseTestFactory.GRPC_DOCUMENT);
		}

		@Test
		void shouldReturnDocument() {
			var result = callService();

			assertThat(result).isEqualTo(document);
		}

		private Document callService() {
			return service.getDocument(DocumentTestFactory.ID);
		}
	}
}
