package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;

class OrganisationsEinheitHeaderModelAssemblerTest {

	@Spy
	private OrganisationsEinheitHeaderModelAssembler assembler;

	@Nested
	class TestToModel {

		@Test
		void shouldHaveSelfLink() {
			var model = assembler.toModel(OrganisationsEinheitHeaderTestFactory.create());

			assertHaveSelfLink(model);
		}

		private void assertHaveSelfLink(EntityModel<OrganisationsEinheitHeader> model) {
			assertThat(model.getLink(IanaLinkRelations.SELF_VALUE)).isPresent().get().extracting(Link::getHref)
					.isEqualTo(OrganisationsEinheitController.PATH + "/" + OrganisationsEinheitHeaderTestFactory.ID);
		}
	}

	@Nested
	class TestToCollectionModel {

		private OrganisationsEinheitHeader organisationsEinheitHeader = OrganisationsEinheitHeaderTestFactory.create();

		@Test
		void shouldCallToModel() {
			callAssembler();

			verify(assembler).toModel(organisationsEinheitHeader);
		}

		@Test
		void shouldHaveSelfLink() {
			var model = callAssembler();

			assertThat(model.getLink(IanaLinkRelations.SELF_VALUE)).isPresent().get().extracting(Link::getHref)
					.isEqualTo(OrganisationsEinheitController.PATH);
		}

		@Test
		void shouldContainEntityModel() {
			var entityModel = EntityModel.of(organisationsEinheitHeader);
			doReturn(entityModel).when(assembler).toModel(organisationsEinheitHeader);

			var model = callAssembler();

			assertThat(model).containsExactly(entityModel);
		}

		private CollectionModel<EntityModel<OrganisationsEinheitHeader>> callAssembler() {
			return assembler.toCollectionModel(List.of(organisationsEinheitHeader));
		}
	}
}
