/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.attacheditem;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.callcontext.CallContextTestFactory;
import de.ozgcloud.alfa.common.callcontext.ContextService;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandBodyMapper;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.kommentar.Kommentar;
import de.ozgcloud.alfa.kommentar.KommentarTestFactory;
import de.ozgcloud.alfa.loeschanforderung.LoeschAnforderung;
import de.ozgcloud.alfa.loeschanforderung.LoeschAnforderungTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.wiedervorlage.Wiedervorlage;
import de.ozgcloud.alfa.wiedervorlage.WiedervorlageTestFactory;

class VorgangAttachedItemServiceTest {

	@Spy
	@InjectMocks
	private VorgangAttachedItemService service;
	@Mock
	private CommandBodyMapper commandBodyMapper;
	@Mock
	private CommandService commandService;
	@Mock
	private VorgangAttachedItemRemoteService vorgangAttachedItemRemoteService;

	@Mock
	private ContextService contextService;

	@DisplayName("Create new wiedervorlage")
	@Nested
	class TestCreateNewWiedervorlage {

		private Wiedervorlage wiedervorlage = WiedervorlageTestFactory.create();

		@BeforeEach
		void mockServices() {
			doReturn(VorgangAttachedItemTestFactory.create()).when(service).buildVorgangAttachedItem(any(), any(), any());
			doReturn(CommandTestFactory.createCreateCommand()).when(service).buildCreateCommand(any(), any());
		}

		@Test
		void shouldBuildVorgangAttachedItem() {
			callCreateNewWiedervorlage();

			verify(service).buildVorgangAttachedItem(wiedervorlage, VorgangHeaderTestFactory.ID, VorgangAttachedItemService.WIEDERVORLAGE_ITEM_NAME);
		}

		@Test
		void shouldBuildCreateCommand() {
			callCreateNewWiedervorlage();

			verify(service).buildCreateCommand(eq(VorgangHeaderTestFactory.ID), any(VorgangAttachedItem.class));
		}

		@Test
		void shouldCallCommandService() {
			callCreateNewWiedervorlage();

			verify(commandService).createCommandDeprecated(any(CreateCommand.class));
		}

		private Command callCreateNewWiedervorlage() {
			return service.createNewWiedervorlage(wiedervorlage, VorgangHeaderTestFactory.ID);
		}
	}

	@DisplayName("Edit wiedervorlage")
	@Nested
	class TestEditWiedervorlage {

		private Wiedervorlage wiedervorlage = WiedervorlageTestFactory.create();

		@BeforeEach
		void mockService() {
			doReturn(VorgangAttachedItemTestFactory.create()).when(service).buildVorgangAttachedItem(any(), any(), any());
			doReturn(CommandTestFactory.createCreateCommand()).when(service).buildUpdateCommand(any(), any(), any());
		}

		@Test
		void shouldBuildVorgangAttachedItem() {
			callEditWiedervorlage();

			verify(service).buildVorgangAttachedItem(wiedervorlage, VorgangHeaderTestFactory.ID, VorgangAttachedItemService.WIEDERVORLAGE_ITEM_NAME);
		}

		@Test
		void shouldBuildCreateCommand() {
			callEditWiedervorlage();

			verify(service).buildUpdateCommand(eq(VorgangHeaderTestFactory.ID), eq(WiedervorlageTestFactory.ID), any(VorgangAttachedItem.class));
		}

		@Test
		void shouldCallCommandService() {
			callEditWiedervorlage();

			verify(commandService).createCommand(any(CreateCommand.class), eq(WiedervorlageTestFactory.VERSION));
		}

		private Command callEditWiedervorlage() {
			return service.editWiedervorlage(wiedervorlage, wiedervorlage.getId(), wiedervorlage.getVersion());
		}
	}

	@Nested
	class TestWiedervorlageDone {
		private Wiedervorlage wiedervorlage = WiedervorlageTestFactory.create();

		@Test
		void shouldCallCommandService() {
			callSetWiedervorlageDone(true);

			verify(commandService).createCommand(any(CreateCommand.class), eq(WiedervorlageTestFactory.VERSION));
		}

		@Test
		void shouldSetItemName() {
			var command = service.createPatchCommand(WiedervorlageTestFactory.create(), false);

			assertThat(((VorgangAttachedItem) command.getBody()).getItemName()).isEqualTo(Wiedervorlage.class.getSimpleName());
		}

		@Test
		void shouldSetItem() {
			var command = service.createPatchCommand(WiedervorlageTestFactory.create(), false);

			assertThat(((VorgangAttachedItem) command.getBody()).getItem()).isNotEmpty();
		}

		@Test
		void shouldSetItemDoneValue() {
			var command = service.createPatchCommand(WiedervorlageTestFactory.create(), true);

			var attachedItem = ((VorgangAttachedItem) command.getBody()).getItem();
			assertThat(attachedItem).containsKey("done");
			assertThat(attachedItem.get("done")).isEqualTo(Boolean.TRUE);
		}

		private Command callSetWiedervorlageDone(boolean done) {
			return service.setWiedervorlageDone(wiedervorlage, done);
		}

	}

	@DisplayName("Create new kommentar")
	@Nested
	class TestCreateNewKommentar {

		private Kommentar kommentar = KommentarTestFactory.create();

		@BeforeEach
		void mockServices() {
			doReturn(VorgangAttachedItemTestFactory.create()).when(service).buildVorgangAttachedItem(any(), any(), any());
			doReturn(CommandTestFactory.createCreateCommand()).when(service).buildCreateCommand(any(), any());
		}

		@Test
		void shouldBuildVorgangAttachedItem() {
			callCreateNewKommentar();

			verify(service).buildVorgangAttachedItem(kommentar, VorgangHeaderTestFactory.ID, VorgangAttachedItemService.KOMMENTAR_ITEM_NAME);
		}

		@Test
		void shouldBuildCreateCommand() {
			callCreateNewKommentar();

			verify(service).buildCreateCommand(eq(VorgangHeaderTestFactory.ID), any(VorgangAttachedItem.class));
		}

		@Test
		void shouldCallCommandService() {
			callCreateNewKommentar();

			verify(commandService).createCommandDeprecated(any(CreateCommand.class));
		}

		private Command callCreateNewKommentar() {
			return service.createNewKommentar(kommentar, VorgangHeaderTestFactory.ID);
		}
	}

	@DisplayName("Edit kommentar")
	@Nested
	class TestEditKommentar {

		private Kommentar kommentar = KommentarTestFactory.create();

		@BeforeEach
		void mockService() {
			doReturn(VorgangAttachedItemTestFactory.create()).when(service).buildVorgangAttachedItem(any(), any(), any());
			doReturn(CommandTestFactory.createCreateCommand()).when(service).buildUpdateCommand(any(), any(), any());
		}

		@Test
		void shouldBuildVorgangAttachedItem() {
			callEditKommentar();

			verify(service).buildVorgangAttachedItem(kommentar, VorgangHeaderTestFactory.ID, VorgangAttachedItemService.KOMMENTAR_ITEM_NAME);
		}

		@Test
		void shouldBuildCreateCommand() {
			callEditKommentar();

			verify(service).buildUpdateCommand(eq(VorgangHeaderTestFactory.ID), eq(KommentarTestFactory.ID), any(VorgangAttachedItem.class));
		}

		@Test
		void shouldCallCommandService() {
			callEditKommentar();

			verify(commandService).createCommand(any(CreateCommand.class), eq(KommentarTestFactory.VERSION));
		}

		private Command callEditKommentar() {
			return service.editKommentar(kommentar, KommentarTestFactory.ID, KommentarTestFactory.VERSION);
		}
	}

	@DisplayName("build vorgang attached item")
	@Nested
	class TestBuildVorgangAttachedItem {

		private static final Wiedervorlage BODY = WiedervorlageTestFactory.create();

		@BeforeEach
		void mockServices() {
			when(commandBodyMapper.fromObjectToMap(any())).thenReturn(VorgangAttachedItemTestFactory.ITEM);
			when(contextService.getClientName()).thenReturn(CallContextTestFactory.CLIENT_NAME);
		}

		@Test
		void shouldCallCommandBodyMapper() {
			buildVorgangAttachedItem();

			verify(commandBodyMapper).fromObjectToMap(BODY);
		}

		@Test
		void shouldContainsVorgangId() {
			var vorgangAttachedItem = buildVorgangAttachedItem();

			assertThat(vorgangAttachedItem.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldContainsItemName() {
			var vorgangAttachedItem = buildVorgangAttachedItem();

			assertThat(vorgangAttachedItem.getItemName()).isEqualTo(VorgangAttachedItemService.WIEDERVORLAGE_ITEM_NAME);
		}

		@Test
		void shouldContainsItem() {
			var vorgangAttachedItem = buildVorgangAttachedItem();

			assertThat(vorgangAttachedItem.getItem()).isEqualTo(VorgangAttachedItemTestFactory.ITEM);
		}

		@Test
		void shouldContainsClientName() {
			var vorgangAttachedItem = buildVorgangAttachedItem();

			assertThat(vorgangAttachedItem.getClient()).isEqualTo(CallContextTestFactory.CLIENT_NAME);
		}

		private VorgangAttachedItem buildVorgangAttachedItem() {
			return service.buildVorgangAttachedItem(BODY, VorgangHeaderTestFactory.ID,
					VorgangAttachedItemService.WIEDERVORLAGE_ITEM_NAME);
		}
	}

	@DisplayName("build createCommand")
	@Nested
	class TestBuildCreateCommand {

		@Test
		void shouldContainsVorgangId() throws Exception {
			var createCommand = buildCommand();

			assertThat(createCommand.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldContainsRelationId() throws Exception {
			var createCommand = buildCommand();

			assertThat(createCommand.getRelationId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldContainsOrder() throws Exception {
			var createCommand = buildCommand();

			assertThat(createCommand.getCommandOrder()).isEqualTo(CommandOrder.CREATE_ATTACHED_ITEM);
		}

		@Test
		void shouldContainsBody() throws Exception {
			var createCommand = buildCommand();

			assertThat(((VorgangAttachedItem) createCommand.getBody())).usingRecursiveComparison()
					.isEqualTo(VorgangAttachedItemTestFactory.create());
		}

		private CreateCommand buildCommand() {
			return service.buildCreateCommand(VorgangHeaderTestFactory.ID, VorgangAttachedItemTestFactory.create());
		}
	}

	@Nested
	class TestCreateNewLoeschAnforderung {

		private final LoeschAnforderung itemBody = LoeschAnforderungTestFactory.create();
		private final CreateCommand createCommand = CommandTestFactory.createCreateCommand();
		private final Command command = CommandTestFactory.create();
		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();

		@BeforeEach
		void init() {
			doReturn(vorgangAttachedItem).when(service)
					.buildVorgangAttachedItem(itemBody, VorgangHeaderTestFactory.ID, VorgangAttachedItemService.LOESCH_ANFORDERUNG_ITEM_NAME);
			doReturn(createCommand).when(service).buildCreateCommand(VorgangHeaderTestFactory.ID, vorgangAttachedItem);
			when(commandService.createCommandDeprecated(createCommand)).thenReturn(command);
		}

		@Test
		void shouldBuildVorgangAttachedItem() {
			callCreateVorgangAttachedItem();

			verify(service).buildVorgangAttachedItem(itemBody, VorgangHeaderTestFactory.ID, VorgangAttachedItemService.LOESCH_ANFORDERUNG_ITEM_NAME);
		}

		@Test
		void shouldBuildCreateCommand() {
			callCreateVorgangAttachedItem();

			verify(service).buildCreateCommand(VorgangHeaderTestFactory.ID, vorgangAttachedItem);
		}

		@Test
		void shouldCallCommandService() {
			callCreateVorgangAttachedItem();

			verify(commandService).createCommandDeprecated(createCommand);
		}

		@Test
		void shouldReturnCommand() {
			var result = callCreateVorgangAttachedItem();

			assertThat(result).isEqualTo(command);
		}

		private Command callCreateVorgangAttachedItem() {
			return service.createNewLoeschAnforderung(itemBody, VorgangHeaderTestFactory.ID);
		}

	}

	@Nested
	@DisplayName("GetById")
	class TestGetById {

		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();

		@Test
		void shouldCallRemoteService() {
			service.getById(VorgangAttachedItemTestFactory.ID);

			verify(vorgangAttachedItemRemoteService).getById(VorgangAttachedItemTestFactory.ID);
		}

		@Test
		void shouldReturnResult() {
			doReturn(vorgangAttachedItem).when(service).getById(VorgangAttachedItemTestFactory.ID);

			var result = service.getById(VorgangAttachedItemTestFactory.ID);

			assertThat(result).isEqualTo(vorgangAttachedItem);
		}
	}

	@Nested
	class TestIsLoeschAnforderung {

		@Test
		void shouldReturnTrue() {
			var isLoeschAnforderung = service.isLoeschAnforderung(VorgangAttachedItemService.LOESCH_ANFORDERUNG_ITEM_NAME);

			assertThat(isLoeschAnforderung).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var isLoeschAnforderung = service.isLoeschAnforderung(VorgangAttachedItemService.WIEDERVORLAGE_ITEM_NAME);

			assertThat(isLoeschAnforderung).isFalse();
		}
	}

	@Nested
	class TestDeleteAttachedItem {

		private final CreateCommand createCommand = CommandTestFactory.createCreateCommand();

		@Captor
		private ArgumentCaptor<CreateCommand> createCommandArgumentCaptor;

		private final Command command = CommandTestFactory.create();
		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.create();

		@BeforeEach
		void init() {
			doReturn(createCommand).when(service).buildDeleteCommand(vorgangAttachedItem);
			when(commandService.createCommand(createCommand, VorgangAttachedItemTestFactory.VERSION)).thenReturn(command);
		}

		@Test
		void shouldBuildCreateCommand() {
			service.deleteAttachedItem(vorgangAttachedItem);

			verify(service).buildDeleteCommand(vorgangAttachedItem);
		}

		@Test
		void shouldCallCommandService() {
			service.deleteAttachedItem(vorgangAttachedItem);

			verify(commandService).createCommand(createCommandArgumentCaptor.capture(), eq(VorgangAttachedItemTestFactory.VERSION));
			assertThat(createCommandArgumentCaptor.getValue()).isEqualTo(createCommand);
		}

		@Test
		void shouldReturnCommand() {
			var resultCommand = service.deleteAttachedItem(vorgangAttachedItem);

			assertThat(resultCommand).isEqualTo(command);
		}
	}

	@DisplayName("Build deleteCommand")
	@Nested
	class TestBuildDeleteCommand {

		private final VorgangAttachedItem item = VorgangAttachedItemTestFactory.create();

		@Test
		void shouldContainsVorgangId() {
			var createCommand = service.buildDeleteCommand(item);

			assertThat(createCommand.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldContainsRelationId() {
			var createCommand = service.buildDeleteCommand(item);

			assertThat(createCommand.getRelationId()).isEqualTo(VorgangAttachedItemTestFactory.ID);
		}

		@Test
		void shouldContainsRelationVersion() {
			var createCommand = service.buildDeleteCommand(item);

			assertThat(createCommand.getRelationVersion()).isEqualTo(VorgangAttachedItemTestFactory.VERSION);
		}

		@Test
		void shouldContainsOrder() {
			var createCommand = service.buildDeleteCommand(item);

			assertThat(createCommand.getCommandOrder()).isEqualTo(CommandOrder.DELETE_ATTACHED_ITEM);
		}

		@Test

		void shouldContainsBody() {
			var createCommand = service.buildDeleteCommand(item);

			assertThat(createCommand.getBody()).usingRecursiveComparison().isEqualTo(item);
		}
	}

	@DisplayName("Find loeschAnforderung")
	@Nested
	class TestFindLoeschAnforderung {

		@Test
		void shouldCallRemoteService() {
			service.findLoeschAnforderung(VorgangHeaderTestFactory.ID);

			verify(vorgangAttachedItemRemoteService).findVorgangAttachedItem(VorgangHeaderTestFactory.ID,
					VorgangAttachedItemService.LOESCH_ANFORDERUNG_ITEM_NAME);
		}

		@Test
		void shouldReturnValue() {
			var vorgangAttachedItemStream = Stream.of(VorgangAttachedItemTestFactory.create());
			when(vorgangAttachedItemRemoteService.findVorgangAttachedItem(anyString(), anyString())).thenReturn(vorgangAttachedItemStream);

			var loeschAnforderungen = service.findLoeschAnforderung(VorgangHeaderTestFactory.ID);

			assertThat(loeschAnforderungen).isEqualTo(vorgangAttachedItemStream);
		}
	}
}