package de.ozgcloud.alfa.loeschanforderung;

import static org.assertj.core.api.Assertions.*;

import java.util.Map;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemTestFactory;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class LoeschAnforderungMapperTest {

	private final LoeschAnforderungMapper mapper = new LoeschAnforderungMapper();

	@DisplayName("From vorgangAttachedItem")
	@Nested
	class TestFromVorgangAttachedItem {

		private final VorgangAttachedItem vorgangAttachedItem = VorgangAttachedItemTestFactory.createBuilder().item(Map.of(
				LoeschAnforderung.REQUESTED_BY_FIELD,
				LoeschAnforderungTestFactory.REQUESTED_BY.toString(),
				LoeschAnforderung.PREV_STATUS_FIELD,
				LoeschAnforderungTestFactory.PREV_STATUS.name())).build();

		@Test
		void shouldMapId() {
			var loeschAnforderung = mapFromVorgangAttachedItem();

			assertThat(loeschAnforderung.getId()).isEqualTo(VorgangAttachedItemTestFactory.ID);
		}

		@Test
		void shouldMapVorgangId() {
			var loeschAnforderung = mapFromVorgangAttachedItem();

			assertThat(loeschAnforderung.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldMapRequestBy() {
			var loeschAnforderung = mapFromVorgangAttachedItem();

			assertThat(loeschAnforderung.getRequestedBy()).isEqualTo(LoeschAnforderungTestFactory.REQUESTED_BY);
		}

		@Test
		void shouldMapPrevStatus() {
			var loeschAnforderung = mapFromVorgangAttachedItem();

			assertThat(loeschAnforderung.getPrevStatus()).isEqualTo(LoeschAnforderungTestFactory.PREV_STATUS);
		}

		private LoeschAnforderung mapFromVorgangAttachedItem() {
			return mapper.fromVorgangAttachedItem(vorgangAttachedItem);
		}
	}

	@DisplayName("From Command")
	@Nested
	class TestBuildLoeschAnforderung {

		private final Command command = CommandTestFactory.createBuilder()
				.body(Map.of(VorgangAttachedItem.ITEM_FIELD, LoeschAnforderungTestFactory.asMap()))
				.build();

		@Test
		void shouldHaveId() {
			var loeschAnforderung = mapper.fromCommand(command);

			assertThat(loeschAnforderung.getId()).isEqualTo(CommandTestFactory.RELATION_ID);
		}

		@Test
		void shouldHaveVorgangId() {
			var loeschAnforderung = mapper.fromCommand(command);

			assertThat(loeschAnforderung.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldHaveRequestedBy() {
			var loeschAnforderung = mapper.fromCommand(command);

			assertThat(loeschAnforderung.getRequestedBy()).isEqualTo(LoeschAnforderungTestFactory.REQUESTED_BY);
		}

		@Test
		void shouldHavePrevStatus() {
			var loeschAnforderung = mapper.fromCommand(command);

			assertThat(loeschAnforderung.getPrevStatus()).isEqualTo(LoeschAnforderungTestFactory.PREV_STATUS);
		}
	}
}
