/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import static org.assertj.core.api.Assertions.*;

import java.time.ZonedDateTime;

import org.junit.jupiter.api.Test;

class PostfachNachrichtComparatorTest {

	private PostfachNachrichtComparator comparator = new PostfachNachrichtComparator();

	@Test
	void shouldCompareCreatedAt() {
		var shouldBeFirst = buildPostfachMail(ZonedDateTime.parse("2020-02-01T01:00:00Z"), null);
		var shouldBeSecond = buildPostfachMail(ZonedDateTime.parse("2020-01-01T01:00:00Z"), null);

		var result = comparator.compare(shouldBeSecond, shouldBeFirst);

		assertThat(result).isEqualTo(-1);
	}

	@Test
	void shouldCompareCreatedAtWithSentAtNull() {
		var shouldBeFirst = buildPostfachMail(ZonedDateTime.parse("2020-02-01T01:00:00Z"), ZonedDateTime.parse("2020-05-01T01:00:00Z"));
		var shouldBeSecond = buildPostfachMail(ZonedDateTime.parse("2020-01-01T01:00:00Z"), null);

		var result = comparator.compare(shouldBeSecond, shouldBeFirst);

		assertThat(result).isEqualTo(-1);
	}

	@Test
	void shouldCompareSentAt() {
		var shouldBeFirst = buildPostfachMail(ZonedDateTime.parse("2020-05-01T01:00:00Z"), ZonedDateTime.parse("2020-10-01T01:00:00Z"));
		var shouldBeSecond = buildPostfachMail(ZonedDateTime.parse("2020-10-01T01:00:00Z"), ZonedDateTime.parse("2020-05-01T01:00:00Z"));

		var result = comparator.compare(shouldBeSecond, shouldBeFirst);

		assertThat(result).isEqualTo(-1);
	}

	@Test
	void shouldCompareSentAtWithCreateAtNull() {
		var shouldBeFirst = buildPostfachMail(ZonedDateTime.parse("2020-01-01T01:00:00Z"), ZonedDateTime.parse("2020-10-01T01:00:00Z"));
		var shouldBeSecond = buildPostfachMail(null, ZonedDateTime.parse("2020-05-01T01:00:00Z"));

		var result = comparator.compare(shouldBeSecond, shouldBeFirst);

		assertThat(result).isEqualTo(-1);
	}

	private PostfachMail buildPostfachMail(ZonedDateTime createdAt, ZonedDateTime sentAt) {
		return PostfachMailTestFactory.createBuilder().createdAt(createdAt).sentAt(sentAt).build();
	}
}