/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.system;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.vorgang.system.GrpcGetSystemStatusRequest;
import de.ozgcloud.vorgang.system.SystemStatusServiceGrpc.SystemStatusServiceBlockingStub;

class SystemStatusRemoteServiceTest {

	@InjectMocks
	private SystemStatusRemoteService service;

	@Mock
	private SystemStatusServiceBlockingStub serviceStub;
	@Mock
	private RemoteSystemStatusMapper mapper;

	@Nested
	class TestGetSystemStatus {

		private static final GrpcGetSystemStatusRequest SYSTEM_STATUS_REQUEST = GrpcGetSystemStatusRequest.newBuilder().build();
		private static final VorgangManagerSystemStatus VORGANG_MANAGER_SYSTEM_STATUS = VorgangManagerSystemStatusTestFactory.create();

		@BeforeEach
		void init() {
			when(mapper.fromGrpc(any())).thenReturn(VORGANG_MANAGER_SYSTEM_STATUS);
		}

		@Test
		void shouldCallRemoteService() {
			service.getSystemStatus();

			verify(serviceStub).getSystemStatus(SYSTEM_STATUS_REQUEST);
		}

		@Test
		void shouldGetSearchServerStatus() {
			when(serviceStub.getSystemStatus(any())).thenReturn(GrpcGetSystemStatusResponseTestFactory.create());

			var vorgangManagerSystemStatus = service.getSystemStatus();

			assertThat(vorgangManagerSystemStatus).isEqualTo(VORGANG_MANAGER_SYSTEM_STATUS);
		}
	}
}